//	Title: Interactive Vehicle Support
//	Author: Trigun
//		BL_ID: 2072
//		Email: trigun@centhra.com
//	Last Revision: 09/19/2012
//
//	Description:
//		This support script provides the basis for allowing
//	the use of interactive objects on a vehicle.  Please,
//	contact the author concerning the discovery of any bugs.
//
//	Perceived Usage:
//		Opening doors on vehicles by activating them.
//
//	Debug Mode:
//		$Debug::InteractiveVehicles = true;
//

package Support_Interactive_Vehicle
{
	function Player::activateStuff(%this)
	{
		for(%a = 0; %a < 1; %a++)
		{
			if(!%this.isMounted())
				break;

			%obj = %this.getObjectMount();
			%db = %obj.getDatablock();
			if(!%db.isInteractive)
				break;

			%posA = %this.getEyePoint();
			%posB = VectorAdd(VectorScale(VectorNormalize(%this.getEyeVector()), 5), %posA);
			%posC = VectorAdd(VectorScale(VectorNormalize(%this.getEyeVector()), 10), %posA);

			%brickCheck = containerRayCast(%posA, %posB, $TypeMasks::FXBrickObjectType, %this);
			if(isObject(%brickCheck))
			{
				%brick = getWord(%brickCheck, 0);

				for(%b = 0; %b < %brick.numEvents; %b++)
					if(%brick.eventEnabled[%b] && %brick.eventInput[%b] $= "onActivate")
						%activate++;

				if(%activate)
					break;

				%newDist = VectorDist(getWords(%brickCheck, 1, 3), %posA);
				%posC = VectorAdd(VectorScale(VectorNormalize(%this.getEyeVector()), %newDist), %posA);
			}

			%vehicleCheck = containerRayCast(%posA, %posC, $TypeMasks::VehicleObjectType, %this);
			if(isObject(%vehicleCheck))
				break;
			
			%veh = containerRayCast(%posC, %posA, $TypeMasks::VehicleObjectType, %this);
			if(!isObject(%veh))
				break;
			if(%obj != %veh)
				break;

			%pos = getWords(%veh, 1, 3);
			%vector = VectorNormalize(VectorSub(%posA, %posC));

			%scale = %veh.getScale();
			%offset = VectorSub(%pos, %veh.getPosition());
			%offset = getWord(%offset, 0) / getWord(%scale, 0) SPC getWord(%offset, 1) / getWord(%scale, 1) SPC getWord(%offset, 2) / getWord(%scale, 2);

			%transform = %veh.getTransform();
			%rotationMatrix = "0 0 0" SPC getWords(%transform, 3, 5) SPC -1 * getWord(%transform, 6);
			%newOffset = MatrixMulVector(%rotationMatrix, %offset);

			if(getWord(%veh, 0).onInteract(%this.client, %newOffset, %pos, %vector))
				return 0;
		}

		parent::activateStuff(%this);
	}

	function Vehicle::onActivate(%this, %obj, %client, %position, %vector)
	{
		%db = %this.getDatablock();
		if(%db.isInteractive)
		{
			%scale = %this.getScale();
			%offset = VectorSub(%position, %this.getPosition());
			%offset = getWord(%offset, 0) / getWord(%scale, 0) SPC getWord(%offset, 1) / getWord(%scale, 1) SPC getWord(%offset, 2) / getWord(%scale, 2);

			%transform = %this.getTransform();
			%rotationMatrix = "0 0 0" SPC getWords(%transform, 3, 5) SPC -1 * getWord(%transform, 6);
			%newOffset = MatrixMulVector(%rotationMatrix, %offset);

			if(%this.onInteract(%client, %newOffset, %position, %vector))
				return 0;
		}

		parent::onActivate(%this, %obj, %client, %position, %vector);
	}
	function Vehicle::onInteract(%this, %client, %offset, %position, %vector)
	{
		%db = %this.getDatablock();
		if(!%db.isInteractive)
			return 0;

		%x = getWord(%offset, 0);
		%y = getWord(%offset, 1);
		%z = getWord(%offset, 2);

		if($Debug::InteractiveVehicles)
		{
			if(isObject(%client))
			{
				echo(%client.getSimpleName() SPC "->" SPC %offset);
				centerPrint(%client, "" NL %offset, 10, 3);
			}
			else
				echo(%offset);
		}

		%bg = getBrickGroupFromObject(%this);
		%canUse = minigameCanUse(%client, %this);
		%time = getSimTime();

		for(%a = 0; %a < %db.iNumZones; %a++)
		{
			%zone = %db.iZone[%a];
			%check = %x >= getWord(%zone, 0) && %x <= getWord(%zone, 3) && %y >= getWord(%zone, 1) && %y <= getWord(%zone, 4) && %z >= getWord(%zone, 2) && %z <= getWord(%zone, 5);

			if(%check)
			{
				if(!%canUse)
				{
					if(isObject(%client.minigame))
						centerPrint(%client, "This vehicle is not part of the mini-game.", 1, 2);
					else
						centerPrint(%client, "This vehicle is part of a mini-game.", 1, 2);
					return -1;
				}

				%cmd = %db @ "." @ %db.iZoneScript[%a] @ "(" @ %this @ ", " @ %client @ ", \"" @ %offset @ "\", \"" @ %position @ "\", \"" @ %vector @ "\");";
				return eval(%cmd);
			}
		}

		return 0;
	}
	function ShapeBase::setNodeColor(%obj, %node, %color)
	{
		parent::setnodecolor(%obj, %node, %color);

		if(%obj.iNumObjects)
			for(%a = 0; %a < %obj.iNumObjects; %a++)
				if(isObject(%obj.iObject[%a]))
					%obj.iObject[%a].setNodeColor("ALL", %color);
	}
};
schedule(0, 0, activatePackage, Support_Interactive_Vehicle);
